<?php
/* +**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 * Contributor(s): YetiForce.com
 * ********************************************************************************** */
require_once dirname(__FILE__) . '/../ModTracker.php';
require_once 'include/events/VTEntityDelta.php';

class ModTrackerHandler extends VTEventHandler
{

	function handleEvent($eventName, $data)
	{
		$adb = PearDatabase::getInstance();
		$log = LoggerManager::getInstance();
		
		if (!is_object($data)) {
			$extendedData = $data;
			$data = $extendedData['entityData'];
		}
		$moduleName = $data->getModuleName();
		$flag = ModTracker::isTrackingEnabledForModule($moduleName);

		if ($flag) {
			$currentUser = Users_Record_Model::getCurrentUserModel();
			
			switch ($eventName) {
				case 'vtiger.entity.aftersave.final':

					$recordId = $data->getId();
					$columnFields = $data->getData();
					$vtEntityDelta = new VTEntityDelta();
					$delta = $vtEntityDelta->getEntityDelta($moduleName, $recordId, true);

					$newerEntity = $vtEntityDelta->getNewEntity($moduleName, $recordId);
					$newerColumnFields = $newerEntity->getData();
					$newerColumnFields = array_change_key_case($newerColumnFields, CASE_LOWER);
					$delta = array_change_key_case($delta, CASE_LOWER);
					if (is_array($delta)) {
						$inserted = false;
						foreach ($delta as $fieldName => $values) {
							if ($fieldName != 'modifiedtime') {
								if (!$inserted) {
									$checkRecordPresentResult = $adb->pquery('SELECT * FROM vtiger_modtracker_basic WHERE crmid = ?', array($recordId));
									if (!$adb->num_rows($checkRecordPresentResult) && $data->isNew()) {
										$status = ModTracker::$CREATED;
									} else {
										$status = ModTracker::$UPDATED;
									}
									$this->id = $adb->getUniqueId('vtiger_modtracker_basic');
									$adb->insert('vtiger_modtracker_basic', [
										'id' => $this->id,
										'crmid' => $recordId,
										'module' => $moduleName,
										'whodid' => $currentUser->getRealId(),
										'changedon' => $newerColumnFields['modifiedtime'],
										'status' => $status
									]);
									$inserted = true;
								}
								$adb->pquery('INSERT INTO vtiger_modtracker_detail(id,fieldname,prevalue,postvalue) VALUES(?,?,?,?)', Array($this->id, $fieldName, $values['oldValue'], $values['currentValue']));
							}
						}
					}
					$isMyRecord = $adb->pquery('SELECT crmid FROM vtiger_crmentity WHERE smownerid <> ? AND crmid = ?', array($currentUser->getRealId(), $recordId));
					if ($adb->num_rows($isMyRecord) > 0) {
						$adb->pquery("UPDATE vtiger_crmentity SET was_read = 0 WHERE crmid = ?;", array($recordId));
					}

					break;
				case 'vtiger.entity.beforedelete':

					$recordId = $data->getId();
					$columnFields = $data->getData();
					$id = $adb->getUniqueId('vtiger_modtracker_basic');
					$adb->insert('vtiger_modtracker_basic', [
						'id' => $id,
						'crmid' => $recordId,
						'module' => $moduleName,
						'whodid' => $currentUser->getRealId(),
						'changedon' => date('Y-m-d H:i:s', time()),
						'status' => ModTracker::$DELETED
					]);
					$isMyRecord = $adb->pquery('SELECT crmid FROM vtiger_crmentity WHERE smownerid <> ? AND crmid = ?', array($currentUser->getRealId(), $recordId));
					if ($adb->num_rows($isMyRecord) > 0) {
						$adb->pquery("UPDATE vtiger_crmentity SET was_read = 0 WHERE crmid = ?;", array($recordId));
					}

					break;
				case 'vtiger.entity.afterrestore':

					$recordId = $data->getId();
					$columnFields = $data->getData();
					$id = $adb->getUniqueId('vtiger_modtracker_basic');
					$adb->insert('vtiger_modtracker_basic', [
						'id' => $id,
						'crmid' => $recordId,
						'module' => $moduleName,
						'whodid' => $currentUser->getRealId(),
						'changedon' => date('Y-m-d H:i:s', time()),
						'status' => ModTracker::$RESTORED
					]);
					$isMyRecord = $adb->pquery('SELECT crmid FROM vtiger_crmentity WHERE smownerid <> ? AND crmid = ?', array($currentUser->getRealId(), $recordId));
					if ($adb->num_rows($isMyRecord) > 0) {
						$adb->pquery("UPDATE vtiger_crmentity SET was_read = 0 WHERE crmid = ?;", array($recordId));
					}

					break;
				case 'vtiger.entity.link.after':
					ModTracker::linkRelation($extendedData['sourceModule'], $extendedData['sourceRecordId'], $extendedData['destinationModule'], $extendedData['destinationRecordId']);
					break;
				case 'vtiger.entity.unlink.after':
					ModTracker::unLinkRelation($extendedData['sourceModule'], $extendedData['sourceRecordId'], $extendedData['destinationModule'], $extendedData['destinationRecordId']);
					break;
				case 'entity.convertlead.after':
					// TODU
					break;
				case 'vtiger.view.detail.before':
					$adb->insert('vtiger_modtracker_basic', [
						'id' => $adb->getUniqueId('vtiger_modtracker_basic'),
						'crmid' => $data->getId(),
						'module' => $moduleName,
						'whodid' => $currentUser->getRealId(),
						'changedon' => date('Y-m-d H:i:s', time()),
						'status' => ModTracker::$DISPLAYED
					]);
					break;
			}
		}
	}
}
